using Abp.Dependency;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using YGNT.Infrastructure.Model;

namespace YGNT.Infrastructure.Helper
{
    public interface IHttpHelper : ITransientDependency
    {
        Task<HttpResult> PostAsync(string uri, string data);
        Task<HttpResult> PutAsync(string uri, string data);
        Task<HttpResult> DeleteAsync(string uri, string data);
        Task<HttpResult> GetAsync(string uri);
        Task<HttpResult> GetAsync(string uri, string data);
        Task<HttpResult> GetTokenAsync(string uri, string token);
        Task<HttpResult> GetResponse(string url, string token = "");
    }
}
